import cc.arduino.*;
import processing.video.*;
import java.text.SimpleDateFormat;
import java.util.Date;
import processing.serial.*;


// Let's build a twittering cat mat 
// bjoern@stanford.edu 2009-04-18

// Momentary switch underneath cat mat (normally closed)
// triggers status updates sent to Twitter

// based on the following libraries & examples:
// FTP - http://processing.org/hacks/hacks:ftp
// Twitter4J - http://yusuke.homeip.net/twitter4j/en/index.html
// Arduino - http://www.arduino.cc/playground/Interfacing/Processing
// Video Capture - File-Examples-Libraries-Video (Capture)-GettingStartedCapture


//-----TWITTER VARS-------------------
String twitterID = ""; //TODO: fill in your info
String twitterPassword = ""; //TODO: fill in your info

//-----FTP VARS-----------------------
FTPClient ftp;
 
// Populate these variables with the necessary info.
String host = ""; //TODO: fill in your info
String username = ""; //TODO: fill in your info
String password =  ""; //TODO: fill in your info
String remotePath = "/";  //TODO: fill in your info
String localPath = "./";
String URLprefix = "http://myserver.com/dir/"; //TODO: fill in your info
//-----CAMERA CAPTURE VARS--------------
Capture cam;

//-----ARDUINO VARS--------------------
Arduino arduino;
int matPin = 2;
int matCounter=0;
int curMatState = Arduino.HIGH;
int lastMatState = Arduino.HIGH;
int lastMatEvent = Arduino.LOW;
int bounceLimit=100;


void setup() {
  size(320,240);

  // CAMERA If no device is specified, will just use the default.
  cam = new Capture(this, 320, 240);
  
  // ARDUINO
  arduino = new Arduino(this, Arduino.list()[0]); // TODO: may have to adjust [0] to different index!  
  arduino.pinMode(matPin, Arduino.INPUT); //set input pin
  arduino.digitalWrite(matPin, Arduino.HIGH); //enable pull-up
}


void draw() {
  // read current mat state
  pollMatState();
  // wait a bit
  delay(100);
}


// Read current mat state with debounce timer and toggle check
// calls fireMatEvent() when state of mat changed and has been stable for a while
void pollMatState() {
  curMatState = arduino.digitalRead(matPin); 
  
  if(curMatState == lastMatState) {
      //still in same state - incrase bounce counter
      if(matCounter < bounceLimit) {
        matCounter++;
      } else if (matCounter==bounceLimit) {
        //we've debounced enough and are ready to fire
        if(lastMatEvent != curMatState) {
          //only fire if this event is difft from last one
          fireMatEvent(curMatState);
          lastMatEvent = curMatState;
        }
        matCounter++;
      } else if (matCounter > bounceLimit) {
        // event already fired - do nothing
      } 
    } else { 
      //restart count
      matCounter=0;
    }
    lastMatState = curMatState;
}


// Fire a new mat state change event
// this function builds the right tweet,
// potentially captures an image and uploads it,
// and updates the twitter status accordingly
void fireMatEvent(int state) {
  String tweet;
  String stat;
  String filename=null;
  boolean withImage = false;
  
  if(state==Arduino.LOW) {
    //off mat event
    tweet = makeTweet(false,null);
    stat = "off mat";
  } else {
    filename = makeFilename();
    try{ 
      withImage = uploadFile(captureImage(filename));
    } catch (Exception e) {
       println("couldn't take/upload image. tweeting without."); 
    }
    if(withImage) {
      tweet = makeTweet(true,filename);
    } else {
      tweet = makeTweet(true,null);
    }
     stat = "on mat";
  } 
  println(stat);
  try{
      sendTweet(tweet);
  } catch (Exception e) {
     println("caught exception while tweeting");
  }
}


// Create a unique filename based on date & time
// @see http://javatechniques.com/blog/dateformat-and-simpledateformat-examples/
String makeFilename() {
   Date now = new Date();
  SimpleDateFormat format =
            new SimpleDateFormat("yyyyMMdd-HHmmss");
  return (format.format(now)+".jpg");
}

// Capture an image and save it under the passed in filename
// in the sketch data directory
String captureImage(String filename) {

    if (cam.available() == true) {
      cam.read();
      image(cam,0,0);
      save(dataPath(filename));
      print("got image");
      return filename;
    } else {
      return null; //we failed to get image
    }
}



// upload a file via FTP
// returns true if successful, false if there was an error
boolean uploadFile(String filename) {
  //check if we got a filename
  if (filename==null) return false;
  
  //connect to server and push file
  try {
    // set up client
    ftp = new FTPClient();
    ftp.setRemoteHost(host); 
    FTPMessageCollector listener = new FTPMessageCollector();
    ftp.setMessageListener(listener);
 
    // connect
    println ("Connecting");
    ftp.connect();
 
    // login
    println ("Logging in");
    ftp.login(username, password);
 
    // set up passive ASCII transfers
    println ("Setting up passive, Binary transfers");
    ftp.setConnectMode(FTPConnectMode.PASV);
    ftp.setType(FTPTransferType.BINARY);
  
    // copy file to server
    println ("Putting file");
    // TODO: how do i get this path programmatically?
    ftp.put(dataPath(filename), "/"+filename);
 
    // get directory and print it to console
    println ("Directory after put");
    String[] files;
    files = ftp.dir(".", true);
    for (int i = 0; i < files.length; i++)
     println (files[i]);
   
    // Shut down client
    println ("Quitting client");
    ftp.quit();
 
    String messages = listener.getLog();
    println ("Listener log:");
 
    println(messages);
    println ("Test complete");
    return true;
    
  } catch (Exception e) {
 
    println ("Demo failed");
  e.printStackTrace();
    
    return false;
  }  
}

// Send the tweet passed in as String argument
void sendTweet(String msg) {
    //construct a new twitter object
    Twitter twitter = new Twitter(twitterID,twitterPassword);
    try {
      Status status = twitter.update(msg);
      System.out.println("Updated the status to [" + status.getText() + "].");
    } catch (Exception e) {
      print(e.getStackTrace());
    }
}

// Create a new status message based on mat state and captured image
// if filename is non-null, we assume file exists on server
// and add URL to end of message.
String makeTweet(boolean onMat, String filename) {
  String[] onMsgs = {
    "Back on my bed. Time for a nap.", 
    "I'm going back to sleep.",
    "Zzzzzzzzzz."};
  String[] offMsgs = {
    "Yawn...Stretch...time to explore.",
    "Time for food.",
    "I dreamed of birds. Time to go hunting."} ;
  
  if(onMat) {
    if(filename != null) {
      return onMsgs[(int)random(onMsgs.length)]+ URLprefix+filename;
    } else {
      return onMsgs[(int)random(onMsgs.length)];
    }
  } else {
    return offMsgs[(int)random(offMsgs.length)];
  }
    
 
}
